library json_mapper_mobx;

import 'package:dart_json_mapper/json_mapper.dart';
import 'package:dart_json_mapper/type_info.dart';
import 'package:mobx/mobx.dart';

final mobXTypeInfoDecorator = MobXTypeInfoDecorator();

class MobXTypeInfoDecorator extends DefaultTypeInfoDecorator {
  @override
  TypeInfo decorate(TypeInfo typeInfo) {
    final isObservableList = typeInfo.typeName.indexOf("ObservableList<") == 0;
    typeInfo.isIterable = typeInfo.isIterable || isObservableList;
    typeInfo.scalarType = detectScalarType(typeInfo);
    return typeInfo;
  }
}

initializeJsonMapperForMobX() {
  JsonMapper.registerTypeInfoDecorator(mobXTypeInfoDecorator);
  // Value decorators for ObservableList variations
  JsonMapper.registerValueDecorator<ObservableList<DateTime>>(
      (value) => ObservableList<DateTime>.of(value.cast<DateTime>()));
  JsonMapper.registerValueDecorator<ObservableList<String>>(
      (value) => ObservableList<String>.of(value.cast<String>()));
  JsonMapper.registerValueDecorator<ObservableList<num>>(
      (value) => ObservableList<num>.of(value.cast<num>()));
  JsonMapper.registerValueDecorator<ObservableList<int>>(
      (value) => ObservableList<int>.of(value.cast<int>()));
  JsonMapper.registerValueDecorator<ObservableList<double>>(
      (value) => ObservableList<double>.of(value.cast<double>()));
  JsonMapper.registerValueDecorator<ObservableList<bool>>(
      (value) => ObservableList<bool>.of(value.cast<bool>()));
}
