part of json_mapper_mobx.test;

@jsonSerializable
@Json(ignoreNullMembers: true)
class MobX {
  ObservableList<String> stringList = ObservableList<String>();
  ObservableList<num> numList = ObservableList<num>();
  ObservableList<bool> boolList = ObservableList<bool>();
  ObservableList<DateTime> dateTimeList = ObservableList<DateTime>();
  ObservableList<int> intList = ObservableList<int>();
  ObservableList<double> doubleList = ObservableList<double>();

  MobX(
      {this.stringList,
      this.numList,
      this.boolList,
      this.dateTimeList,
      this.doubleList,
      this.intList});
}

testBasics() {
  group("[Verify ObservableList variations for Dart built-in types]", () {
    test("ObservableList<String>", () {
      // given
      final String json =
          '''{"stringList":["aa@test.com","bb@test.com","cc@test.com"]}''';
      MobX m = MobX(
          stringList: ObservableList<String>.of(
              ['aa@test.com', 'bb@test.com', 'cc@test.com']));
      // when
      final String targetJson = JsonMapper.serialize(m, '');
      final MobX instance = JsonMapper.deserialize(targetJson);
      // then
      expect(targetJson, json);
      expect(instance, TypeMatcher<MobX>());
      expect(instance.stringList, TypeMatcher<ObservableList<String>>());
    });

    test("ObservableList<num>", () {
      // given
      final String json = '''{"numList":[1,2.1,3]}''';
      MobX m = MobX(numList: ObservableList<num>.of([1, 2.1, 3]));
      // when
      final String targetJson = JsonMapper.serialize(m, '');
      final MobX instance = JsonMapper.deserialize(targetJson);
      // then
      expect(targetJson, json);
      expect(instance, TypeMatcher<MobX>());
      expect(instance.numList, TypeMatcher<ObservableList<num>>());
    });

    test("ObservableList<int>", () {
      // given
      final String json = '''{"intList":[1,2,3]}''';
      MobX m = MobX(intList: ObservableList<int>.of([1, 2, 3]));
      // when
      final String targetJson = JsonMapper.serialize(m, '');
      final MobX instance = JsonMapper.deserialize(targetJson);
      // then
      expect(targetJson, json);
      expect(instance, TypeMatcher<MobX>());
      expect(instance.intList, TypeMatcher<ObservableList<int>>());
    });

    test("ObservableList<double>", () {
      // given
      final String json = '''{"doubleList":[1.0003,2.0,3.0]}''';
      MobX m = MobX(doubleList: ObservableList<double>.of([1.0003, 2, 3]));
      // when
      final String targetJson = JsonMapper.serialize(m, '');
      final MobX instance = JsonMapper.deserialize(targetJson);
      // then
      expect(targetJson, json);
      expect(instance, TypeMatcher<MobX>());
      expect(instance.doubleList, TypeMatcher<ObservableList<double>>());
    });

    test("ObservableList<bool>", () {
      // given
      final String json = '''{"boolList":[true,false,true]}''';
      MobX m = MobX(boolList: ObservableList<bool>.of([true, false, true]));
      // when
      final String targetJson = JsonMapper.serialize(m, '');
      final MobX instance = JsonMapper.deserialize(targetJson);
      // then
      expect(targetJson, json);
      expect(instance, TypeMatcher<MobX>());
      expect(instance.boolList, TypeMatcher<ObservableList<bool>>());
    });

    test("ObservableList<DateTime>", () {
      // given
      final String json =
          '''{"dateTimeList":["2003-02-28 00:00:00.000","2013-01-23 00:00:00.000","2014-02-20 00:00:00.000"]}''';
      MobX m = MobX(
          dateTimeList: ObservableList<DateTime>.of([
        DateTime.parse('2003-02-28 00:00:00.000'),
        DateTime.parse('2013-01-23 00:00:00.000'),
        DateTime.parse('2014-02-20 00:00:00.000')
      ]));
      // when
      final String targetJson = JsonMapper.serialize(m, '');
      final MobX instance = JsonMapper.deserialize(targetJson);
      // then
      expect(targetJson, json);
      expect(instance, TypeMatcher<MobX>());
      expect(instance.dateTimeList, TypeMatcher<ObservableList<DateTime>>());
    });
  });
}
